﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace furdostat
{
    class IdoPont
    {
        char sep = ':';
        public byte Ora { get; set; }
        public byte Perc { get; set; }
        public byte Mperc { get; set; }
        public uint IdoTick()
        {
            return ((uint)Ora * 60 + Perc) * 60 + Mperc;
        }
        public IdoPont(byte ora, byte perc, byte mperc)
        {
            Ora = ora;
            Perc = perc;
            Mperc = mperc;
        }

        public IdoPont(string idokif)
        {
            string[] resz = idokif.Split(sep);
            Ora = byte.Parse(resz[0]);
            Perc = byte.Parse(resz[1]);
            Mperc = byte.Parse(resz[2]);
        }

        static public bool operator< (IdoPont ez, IdoPont ennel)
        {
            return ez.IdoTick() < ennel.IdoTick();
        }
        static public bool operator> (IdoPont ez, IdoPont ennel)
        {
            return ez.IdoTick() > ennel.IdoTick();
        }
        static public bool operator<= (IdoPont ez, IdoPont ennel)
        {
            return ez.IdoTick() <= ennel.IdoTick();
        }
        static public bool operator >=(IdoPont ez, IdoPont ennel)
        {
            return ez.IdoTick() >= ennel.IdoTick();
        }

        public override string ToString()
        {
            return Ora.ToString() + ":" + Perc.ToString() + ":" + Mperc.ToString();
        }

        static public IdoTartam operator-(IdoPont veg, IdoPont kezd)
        {
            try
            {
                return new IdoTartam(veg.IdoTick() - kezd.IdoTick());
            }
            catch (Exception e)
            {
                Console.Error.WriteLine(e.Message);
                throw;
            }   
        }
    }

    class IdoTartam
    {
        public uint Ido { get; set; }

        public IdoTartam(uint ido)
        {
            Ido = ido;
        }

        public byte Ora()
        {
            return (byte)(Ido / 3600);
        }
        public byte Perc()
        {
            return (byte)((Ido % 3600) / 60);
        }
        public byte Mperc()
        {
            return (byte)(Ido % 60);
        }

        public static bool operator <(IdoTartam it1, IdoTartam it2)
        {
            return (it1.Ido < it2.Ido);
        }
        public static bool operator >(IdoTartam it1, IdoTartam it2)
        {
            return (it1.Ido > it2.Ido);
        }
        public static IdoTartam operator +(IdoTartam it1, IdoTartam it2)
        {
            return new IdoTartam(it1.Ido + it2.Ido);
        }
        public override string ToString()
        {
            return Ora().ToString() + ":" + Perc().ToString() + ":" + Mperc().ToString();
        }
    }

    class Hasznal
    {
        public int Hely { get; set; }
        public IdoPont Kezd { get; set; }
        public IdoPont Veg { get; set; }
        public IdoTartam Hossz { get; set; }
        public Hasznal(int hely, IdoPont kezd, IdoPont veg)
        {
            Hely = hely;
            Kezd = kezd;
            Veg = veg;
            Hossz = veg-kezd;
        }
    }

    class Vendeg
    {
        public int Azon { get; set; }
        public IdoPont Tol { get; set; } //öltözőből kilép
        public IdoPont Ig { get; set; } //öltözőbe belép
        public List<Hasznal> Latogat { get; set; }
        public Vendeg(int azon, IdoPont kezd)
        {
            Azon = azon;
            Tol = kezd;
            Latogat = new List<Hasznal>();
            Ig = new IdoPont(24, 0, 0);
        }
        public IdoTartam Eltolt()
        {
            return Ig-Tol;
        }
    }

    class Program
    {
        static List<Vendeg> vendegek = new List<Vendeg>();

        static void Main(string[] args)
        {
            F1_Input();
            Console.WriteLine("2.feladat");
            Console.WriteLine("Az első vendég {0}-kor lépett ki az öltözőből.",vendegek[0].Tol);
            Console.WriteLine("Az utolsó vendég {0}-kor lépett ki az öltözőből.",vendegek[vendegek.Count - 1].Tol);
            Console.WriteLine("3. feladat");
            Console.WriteLine("A fürdőben {0} vendég járt csak egy részlegen.", F3_Count());
            Console.WriteLine("4. feladat");
            Console.WriteLine("A legtöbb időt eltöltő vendég:");
            int maxhely = F4_Max();
            Console.WriteLine("{0}. vendég {1}", vendegek[maxhely].Azon, vendegek[maxhely].Eltolt());
            Console.WriteLine("5. feladat");
            int[] napszakDb = F5_Count();
            Console.WriteLine("6-9 óra között {0} vendég", napszakDb[0]);
            Console.WriteLine("9-16 óra között {0} vendég", napszakDb[1]);
            Console.WriteLine("16-20 óra között {0} vendég", napszakDb[2]);
            F6_Output();
            Console.WriteLine("7. feladat");
            F7_Map();

            Console.ReadLine();
        }

        static void F1_Input()
        {
            using (System.IO.StreamReader sr = new System.IO.StreamReader("furdoadat.txt"))
            {
                string[] adatsor;
                Vendeg v;
                while (!sr.EndOfStream)
                {
                    adatsor = sr.ReadLine().Split(' ');
                    if (adatsor[1] =="0" && adatsor[2] == "1")
                    {
                        v = new Vendeg(int.Parse(adatsor[0]), new IdoPont(byte.Parse(adatsor[3]), byte.Parse(adatsor[4]), byte.Parse(adatsor[5])));
                        adatsor = sr.ReadLine().Split(' ');
                        while (!(adatsor[1] == "0" && adatsor[2] == "0"))
                        {
                            IdoPont k = new IdoPont(byte.Parse(adatsor[3]), byte.Parse(adatsor[4]), byte.Parse(adatsor[5]));
                            adatsor = sr.ReadLine().Split(' ');
                            Hasznal h = new Hasznal(int.Parse(adatsor[1]), k, new IdoPont(byte.Parse(adatsor[3]), byte.Parse(adatsor[4]), byte.Parse(adatsor[5])));
                            v.Latogat.Add(h);
                            adatsor = sr.ReadLine().Split(' ');
                        }
                            v.Ig.Ora = byte.Parse(adatsor[3]);
                            v.Ig.Perc = byte.Parse(adatsor[4]);
                            v.Ig.Mperc = byte.Parse(adatsor[5]);
                            vendegek.Add(v);
                    }
                   
                }
                
            }
        }

        static public int F3_Count()
        {
            int db = 0;
            foreach (Vendeg v in vendegek)
            {
                if (v.Latogat.Count == 1)
                {
                    db++;
                }
            }
            return db;
        }

        static int F4_Max()
        {
            int max = 0;
            for (int i = 1; i < vendegek.Count; i++)
            {
                if (vendegek[max].Eltolt() < vendegek[i].Eltolt())
                {
                    max = i;
                }
            }
            return max;
        }
        static int[] F5_Count()
        {
            int[] ival = new int[3];
            int i = 0;
            ival[0] = 0;
            ival[1] = 0;
            IdoPont ip1 = new IdoPont("9:0:0");
            IdoPont ip2 = new IdoPont("16:0:0");
            while (vendegek[i].Tol < ip1)
            {
                ival[0]++;
                i++;
            }
            while (vendegek[i].Tol < ip2)
            {
                ival[1]++;
                i++;
            }
            ival[2] = vendegek.Count - i;
        
            return ival;
        }
        
        static void F6_Output()
        {
            using(System.IO.StreamWriter sw = new System.IO.StreamWriter("szauna.txt"))
            {
                foreach (Vendeg v in vendegek)
                {
                    IdoTartam szum = new IdoTartam(0);
                    foreach (Hasznal h in v.Latogat)
                    {
                        if (h.Hely == 2)
                        {
                            szum += h.Hossz;
                        }
                    }
                    if (szum.Ido !=0)
                    {
                        sw.WriteLine("{0} {1}", v.Azon, szum);
                    }

                }
            }
        }
        static void F7_Map()
        {
            string[] helyek = { "Öltöző", "Uszoda", "Szaunák", "Gyógyvizes medencék", "Strand" };
            int[] terekDb = new int[5];
            for (int i = 0; i < 5; i++)
            {
                terekDb[i] = 0;
            }
            foreach (Vendeg v in vendegek)
            {
                bool[] van = new bool[5];
                for (int i = 0; i < 5; i++)
                {
                    van[i] = false;
                }
                //Mivel gyakoribb az ismétlés nélküli és csak 5-féle lehet,
                //nem érdemes vizsgálni, hogy az adott vendég volt-e korábban az adott helyen.
                //A true esetén "megerősítem" a true-t.
                foreach (Hasznal h in v.Latogat)
                {
                    van[h.Hely] = true;
                }
                for (int i = 0; i < 5; i++)
                {
                    terekDb[i] += van[i] ? 1 : 0;
                }
            }
            
            for (int i = 1; i < 5; i++)
            {
                Console.WriteLine("{0}: {1}", helyek[i], terekDb[i]);
            }
        }
    }
}
